function [passedtotalModelRuns, passedtotalParams, failedtotalModelRuns, failedtotalParams] = eval_model_performance(calibrationInput, modelRuns, passCriteria)
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%
% Evaluates model performance by comparing model run outcomes against
% experimental data provided by the user.
%
% INPUTS
%   - calibrationInput: structure that holds user input to calibration
%   program.
%   - modelRuns: structure that holds modelRuns output from the lhs_ode_run
%   function as well as other information such as paramMatrix.
%
% OUTPUTS
%   - passedtotalModelRuns -- a matrix that holds the runs that passed
%   following comparison to user-provided data.
%   - passedtotalParams:  a matrix holding the parameter values for the
%   runs that passed following comparison to user-provided data.
%   - failedtotalModelRuns:  a matrix that holds the runs that failed
%   following comparison to user-provided data. 
%   - failedtotalParams: a matrix that holds the parameter values for the
%   runs that failed following comparison to user-provided data. 
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------

%name some tempvars for model Output:
preyOutput = modelRuns.odeOut{1,1}*1000; % timesteps x number of runs for output 1 -- this is multipled by 1000 because the experimental data is in the thousands
predOutput = modelRuns.odeOut{2,1}*1000; % timesteps x number of runs for output 2 -- this is multipled by 1000 because the experimental data is in the thousands

%name some tempvars for experimental data:
hareDS = calibrationInput.HareDS;
lynxDS = calibrationInput.LynxDS;

% how does the output perform against dataset?
[passedPreyRuns, runIndxPreyPassed, runIndxPreyFailed] = find_passed(preyOutput, hareDS,passCriteria);
[passedPredRuns, runIndxPredPassed, runIndxPredFailed] = find_passed(predOutput, lynxDS,passCriteria);

%find the columns that are in common between the two conditions (prey data
%and predator data)
CommonIndxPassed = intersect(runIndxPreyPassed, runIndxPredPassed);
CommonIndxFailed = union(runIndxPreyFailed, runIndxPredFailed);
%subset runs, LHS matrix, and param matrix to include only the ones that
%passed:
passedtotalParams = modelRuns.paramMatrix(CommonIndxPassed,:);
passedtotalModelRuns(:,:,1) = preyOutput(:,CommonIndxPassed);
passedtotalModelRuns(:,:,2) = predOutput(:,CommonIndxPassed);

%subset runs and param matrix to also include those that did not pass:
failedtotalParams = modelRuns.paramMatrix(CommonIndxFailed,:);
failedtotalModelRuns(:,:,1) = preyOutput(:,CommonIndxFailed);
failedtotalModelRuns(:,:,2) = predOutput(:,CommonIndxFailed);


end


function [passedModelRuns, RunIndxThatPassed, RunIndxThatFailed] = find_passed(modelOutput, expData, passCriteria)


%logical indexing to quickly check if the runs pass through the window of
%our data (time or divided by our passing criteria). 
LogicalArrayCondition1 = modelOutput(:,:) >= expData(:,1) / passCriteria;
LogicalArrayCondition2 = modelOutput(:,:) <= expData(:,2) * passCriteria;
LogicalArraytot = LogicalArrayCondition1 & LogicalArrayCondition2;

% cols with all ones --  meaning they passed the criteria
RunIndxThatPassed = find(all(LogicalArraytot==1)); 

%cols with all zeroes -- meaning they failed the criteria:
RunIndxThatFailed = find(~all(LogicalArraytot));

%use RunIndxThatPassed to pull those columns from the total runs matrix:
passedModelRuns = modelOutput(:,RunIndxThatPassed);


end
