function [hdi, xdensityPass, xPass, xdensityFail, xFail] = find_hdr(passedtotalParamsVec, failedtotalParamsVec, coverage)
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% for a given set of parameter values, this file will find the density
% distribution of those parameter values, and calculate the HDR according
% to the algorithm set forth by Rob Hyndman in 1996. 
%
% Input: 
%    passedtotalParamsVec -- number of passed runs by 1 (the parameter that
%    is being calculated to find the HDR). 
%
%    failedtotalParamsVec -- number of failed runs by 1 (the parameter that
%    is being calculated to find the HDR, the same as the
%    passedtotalParamsVec parameter). 
%
%    coverage -- value that determines the region that we are searching
%    for. Value from 0-1, equal to value calibrationInput.portionParams.
%  
% Output:
%    hdi -- 1 by 2. the highest density interval.  the min and max parameter values
%    that corespond to the highest density region.  These values will be
%    used as the min and max for sampling the parameter range on the next
%    iteration of CaliPro. 
%
%    xdensityPass -- 1 by 100. 1 by 100 set of density values that describe the underlying
%    pass dataset distribution. 
%
%    xPass  -- the xvalues (parameter values) that correspond to the
%    xdensityPass values.  1 by 100.
%
%    xdensityFail -- 1 by 100. 1 by 100 set of density values that describe the underlying
%    fail dataset distribution. 
%
%    xFail -- the xvalues (parameter values) that correspond to the
%    xdensityFail values.  1 by 100.
%-------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------

%find the density estimation
[xdensityPass,xPass] = ks_density_twice(passedtotalParamsVec);

%find the highest density region (returned as the hdi, highest density
%interval) according to Rob Hyndman's algorithm for finding HDR.
best = 0; %counter to hold the best value as you iterate through
for ai = 1:(length(xPass) - 1)
    for bi = (ai + 1):length(xPass)
        mass = sum(diff(xPass(ai:bi)) .* xdensityPass((ai + 1):bi));
        if (mass >= coverage && mass / (xPass(bi) - xPass(ai)) > best)
            best = mass / (xPass(bi) - xPass(ai));
            ai_best = ai;
            bi_best = bi;
        end
    end
end
%return the x ranges for where the best ai_ and bi_ are found
hdi = [xPass(ai_best), xPass(bi_best)]; 

%if there are no failedtotalParamsVec
if (isempty(failedtotalParamsVec) == 1 )
    xdensityFail = []; %make the density empty
    xFail = []; %make the xvalues that go with the density empty too  
else %find the densities for the failed parameters as well    
    [xdensityFail,xFail] = ks_density_twice(failedtotalParamsVec);
end


end

