function CaliProOut = pusheenPlot_pp(dotMatName)
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% pusheenPlot_pp.m is a function file for creating a grid of density plots
% based on the CaliPro output .mat file. By default, it creates a grid of 
% plots with number of rows = number of iterations and number of columns
% equal to the number of parameters varied in CaliPro.
% 
% input: 
%        dotMatName ---- .mat file saved during CaliPro,
%        'ADS_SIMpreypredMinMax_5Iter_after2_1point25Bounds.mat' --
%        this is the CaliPro results from the example outlined
%        in the paper. 
% 
%        within dotMatName, specifically used by this function:       
%             CalibrationOutputArray -- a 1 x Number of Iterations (in this
%             case, the default load file has 5 iterations) cell array that
%             holds the CaliPro output information for each iteration of
%             CaliPro. Within each array index, a data structure holds
%             the information, including the following variables which are
%             accessed by this function file for the pusheen plots:
%
%             .ParamsDensityRangeFail -- number of varied parameters by
%             100. This holds the x values for each of the density plot
%             values that will be plotted for each parameter value that
%             maps to a fail run.
%
%             .ParamsDensityRangePass -- number of varied parameters by
%             100. This holds the x values for each of the density plot
%             values that will be plotted for each parameter value that
%             maps to a pass run.
%
%             .ParamsDensityFail -- number of varied parameters by 100.
%             Matrix of density values for the fail parameter sets.
%
%             .ParamsDensityPass -- number of varied parameters by 100.
%             Matrix of density values for the pass parameter sets.
%
%             .hdrRange -- number of varied parameters by 2. For each
%             parameter, this matrix holds the range, where min is the
%             first column and maximum is the second column, of the next
%             iteration's sampling space.
%
%             .PRCC_var -- 1 by number of varied parameters cell array.
%             Holds the names of each varied parameter. 
%
% output: 
%        CaliProOut --- a data structure that holds all the results
%        from the CaliPro .mat.
% 
%        Figure 1 --- the grid of density plots (referred to as a Pusheen
%        plot) to show, for each parameter, at each iteration, where the
%        pass and fail runs occupy parameter space.
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------

%set the colors for pass and fail:
passBlue = [51/255 187/255 255/255 ];
failRed = [192/255 43/255 32/255];
%set the color that defines the parameter space for the next CaliPro
%iteration:
purpNextIt = [137/255 65/255 249/255];

% load the .mat that holds all the info from CaliPro on the predator prey
% model
%CaliProOut = load('ADS_SIMpreypredMinMax_5Iter_after2_1point25Bounds.mat'); 
CaliProOut = load(dotMatName); 

%create a pusheen plot for each parameter set, for each iteration. subplot1
%is a slightly modified function based on subplot. make it size 5 by 4, for
%this example case, but the following code should run independent of how
%many iterations or how many parameters you vary.  For readability, the
%user might want to break this up. 
subplot1(numel(CaliProOut.CalibrationOutputArray)-1,numel(CaliProOut.CalibrationOutputArray{1}.PRCC_var),'Gap',[0.02 0.03],'XTickL','All');

for j=1:numel(CaliProOut.CalibrationOutputArray)-1 %loop through each iteration
    
    for i=1:numel(CaliProOut.CalibrationOutputArray{1}.PRCC_var) %loop through each varied param
        %set up the index of this iteration's and parameter's plot
        subplot1(i+(numel(CaliProOut.CalibrationOutputArray{1}.PRCC_var)*(j-1)))  
        
        %plot the density plots for fail and pass 
        area(CaliProOut.CalibrationOutputArray{j}.ParamsDensityRangeFail(i,:), ...
            CaliProOut.CalibrationOutputArray{j}.ParamsDensityFail(i,:), ...
            'FaceColor', failRed, 'FaceAlpha', 0.5)
        hold on
        area(CaliProOut.CalibrationOutputArray{j}.ParamsDensityRangePass(i,:),  ...
            CaliProOut.CalibrationOutputArray{j}.ParamsDensityPass(i,:), ...
            'FaceColor', passBlue, 'FaceAlpha', 0.5)
        hold on
        
        %draw the purple line that indicates the range of sampling for the
        %next iteration
        line(CaliProOut.CalibrationOutputArray{j}.hdrRange(i,:), [0 0], ...
            'color', purpNextIt, 'LineWidth', 3)
        
        %if on the final iteration, label the x axis
        if j==numel(CaliProOut.CalibrationOutputArray)-1
            xlabel(CaliProOut.CalibrationOutputArray{1, j}.PRCC_var{i});
        end
        
        %take out all ytick markers. take out any ytick labels. Unnecessary
        %and distracting for the pusheen plot reading.
        yticks([]); 
        yticklabels({});         
    end

end
end
